@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2021 NVIDIA Corporation
+    Copyright (c) 2016-2024 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -22,342 +22,28 @@
 *******************************************************************************/
 
 #include "uvm_hmm.h"
-//
-// You need all of the following in order to actually run HMM:
-//
-//     1) A version of Linux with make_device_exclusive_range() defined
-//        (usually 5.14 or later), with config CONFIG_HMM_MIRROR and
-//        CONFIG_DEVICE_PRIVATE set.
-//
-//     2) *Not* passing UVM Kernel module parameter "disable_hmm=1".
-//
-//     3) ATS must not be enabled
-//
-//     4) UvmInitialize() called without the UVM_INIT_FLAGS_DISABLE_HMM or
-//        UVM_INIT_FLAGS_MULTI_PROCESS_SHARING_MODE flags
-//
-//     5) A system with Turing+ GPUs since GSP OpenRM is required and GPU
-//        VA bits >= CPU VA bits. Otherwise, the GPU can't address all of
-//        the CPU's virtual address space.
-//
-static bool disable_hmm = false;
-module_param(disable_hmm, bool, 0444);
-MODULE_PARM_DESC(disable_hmm,
+
+// Support for HMM ( https://docs.kernel.org/mm/hmm.html ):
+
+#ifdef NVCPU_X86_64
+static bool uvm_disable_hmm = false;
+MODULE_PARM_DESC(uvm_disable_hmm,
                  "Force-disable HMM functionality in the UVM driver. "
-                 "Default: false (i.e, HMM is potentially enabled). Ignored if "
-                 "HMM is not supported in the driver, or if ATS settings "
-                 "conflict with HMM.");
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+                 "Default: false (HMM is enabled if possible). "
+                 "However, even with uvm_disable_hmm=false, HMM will not be "
+                 "enabled if is not supported in this driver build "
+                 "configuration, or if ATS settings conflict with HMM.");
+#else
+// So far, we've only tested HMM on x86_64, so disable it by default everywhere
+// else.
+static bool uvm_disable_hmm = true;
+MODULE_PARM_DESC(uvm_disable_hmm,
+                 "Force-disable HMM functionality in the UVM driver. "
+                 "Default: true (HMM is not enabled on this CPU architecture). "
+                 "However, even with uvm_disable_hmm=false, HMM will not be "
+                 "enabled if is not supported in this driver build "
+                 "configuration, or if ATS settings conflict with HMM.");
+#endif
+
+module_param(uvm_disable_hmm, bool, 0444);
 
