@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -37,9 +37,14 @@
 #include <linux/kernfs.h>
 #endif
 
+#include "detect-self-hosted.h"
+
 #if !defined(NV_BUS_TYPE_HAS_IOMMU_OPS)
 #include <linux/iommu.h>
 #endif
+#if NV_IS_EXPORT_SYMBOL_GPL_pci_ats_supported
+#include <linux/pci-ats.h>
+#endif
 
 static void
 nv_check_and_exclude_gpu(
@@ -462,9 +467,7 @@ next_bar:
     // PPC64LE platform where ATS is currently supported (IBM P9).
     nv_ats_supported &= nv_platform_supports_numa(nvl);
 #else
-#if defined(NV_PCI_DEV_HAS_ATS_ENABLED)
-    nv_ats_supported &= pci_dev->ats_enabled;
-#endif
+    nv_ats_supported = NV_FALSE;
 #endif
     if (nv_ats_supported)
     {
