@@ -1,19 +1,48 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 1999-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #define  __NO_VERSION__
-#include "nv-misc.h"
 
 #include "os-interface.h"
 #include "nv-linux.h"
 
+#if defined(NVCPU_FAMILY_X86) && defined(NV_FOLL_LONGTERM_PRESENT) && \
+    (defined(NV_PIN_USER_PAGES_HAS_ARGS_VMAS) ||                      \
+     defined(NV_GET_USER_PAGES_HAS_ARGS_FLAGS_VMAS))
+#define NV_NUM_PIN_PAGES_PER_ITERATION 0x80000
+#endif
+
+static inline int nv_follow_pfn(struct vm_area_struct *vma,
+                                unsigned long address,
+                                unsigned long *pfn)
+{
+#if defined(NV_FOLLOW_PFN_PRESENT)
+    return follow_pfn(vma, address, pfn);
+#else
+    return -1;
+#endif
+}
+
 /*!
  * @brief Locates the PFNs for a user IO address range, and converts those to
  *        their associated PTEs.
@@ -38,7 +67,7 @@ static NV_STATUS get_io_ptes(struct vm_area_struct *vma,
 
     for (i = 0; i < page_count; i++)
     {
-        if (follow_pfn(vma, (start + (i * PAGE_SIZE)), &pfn) < 0)
+        if (nv_follow_pfn(vma, (start + (i * PAGE_SIZE)), &pfn) < 0)
         {
             return NV_ERR_INVALID_ADDRESS;
         }
@@ -56,7 +85,7 @@ static NV_STATUS get_io_ptes(struct vm_area_struct *vma,
         // user's declaration. Ensure the PFNs represent a contiguous range,
         // error if they do not.
         //
-        if ((*pte_array)[i] != ((*pte_array)[i-1] + PAGE_SIZE))
+        if ((NvU64)pte_array[i] != (((NvU64)pte_array[i-1]) + PAGE_SIZE))
         {
             return NV_ERR_INVALID_ADDRESS;
         }
@@ -64,59 +93,10 @@ static NV_STATUS get_io_ptes(struct vm_area_struct *vma,
     return NV_OK;
 }
 
-/*!
- * @brief Pins user IO pages that have been mapped to the user processes virtual
- *        address space with remap_pfn_range.
- *
- * @param[in]     vma VMA that contains the virtual address range given by the
- *                    start and the page count.
- * @param[in]     start Beginning of the virtual address range of the IO pages.
- * @param[in]     page_count Number of pages to pin from start.
- * @param[in,out] page_array Storage array for pointers to the pinned pages.
- *                           Must be large enough to contain at least page_count
- *                           pointers.
- *
- * @return NV_OK if the pages were pinned successfully, error otherwise.
- */
-static NV_STATUS get_io_pages(struct vm_area_struct *vma,
-                              NvUPtr start,
-                              NvU64 page_count,
-                              struct page **page_array)
-{
-    NV_STATUS rmStatus = NV_OK;
-    NvU64 i, pinned = 0;
-    unsigned long pfn;
-
-    for (i = 0; i < page_count; i++)
-    {
-        if ((follow_pfn(vma, (start + (i * PAGE_SIZE)), &pfn) < 0) ||
-            (!pfn_valid(pfn)))
-        {
-            rmStatus = NV_ERR_INVALID_ADDRESS;
-            break;
-        }
-
-        // Page-backed memory mapped to userspace with remap_pfn_range
-        page_array[i] = pfn_to_page(pfn);
-        get_page(page_array[i]);
-        pinned++;
-    }
-
-    if (pinned < page_count)
-    {
-        for (i = 0; i < pinned; i++)
-            put_page(page_array[i]);
-        rmStatus = NV_ERR_INVALID_ADDRESS;
-    }
-
-    return rmStatus;
-}
-
 NV_STATUS NV_API_CALL os_lookup_user_io_memory(
     void   *address,
     NvU64   page_count,
-    NvU64 **pte_array,
-    void  **page_array
+    NvU64 **pte_array
 )
 {
     NV_STATUS rmStatus;
@@ -158,24 +138,15 @@ NV_STATUS NV_API_CALL os_lookup_user_io_memory(
         goto done;
     }
 
-    if (follow_pfn(vma, start, &pfn) < 0)
+    if (nv_follow_pfn(vma, start, &pfn) < 0)
     {
         rmStatus = NV_ERR_INVALID_ADDRESS;
         goto done;
     }
 
-    if (pfn_valid(pfn))
-    {
-        rmStatus = get_io_pages(vma, start, page_count, (struct page **)result_array);
-        if (rmStatus == NV_OK)
-            *page_array = (void *)result_array;
-    }
-    else
-    {
-        rmStatus = get_io_ptes(vma, start, page_count, (NvU64 **)result_array);
-        if (rmStatus == NV_OK)
-            *pte_array = (NvU64 *)result_array;
-    }
+    rmStatus = get_io_ptes(vma, start, page_count, (NvU64 **)result_array);
+    if (rmStatus == NV_OK)
+        *pte_array = (NvU64 *)result_array;
 
 done:
     nv_mmap_read_unlock(mm);
@@ -198,9 +169,15 @@ NV_STATUS NV_API_CALL os_lock_user_pages(
     NV_STATUS rmStatus;
     struct mm_struct *mm = current->mm;
     struct page **user_pages;
-    NvU64 i, pinned;
-    NvBool write = DRF_VAL(_LOCK_USER_PAGES, _FLAGS, _WRITE, flags), force = 0;
-    int ret;
+    NvU64 i;
+    NvU64 npages = page_count;
+    NvU64 pinned = 0;
+    unsigned int gup_flags = DRF_VAL(_LOCK_USER_PAGES, _FLAGS, _WRITE, flags) ? FOLL_WRITE : 0;
+    long ret;
+
+#if defined(NVCPU_FAMILY_X86) && defined(NV_FOLL_LONGTERM_PRESENT)
+    gup_flags |= FOLL_LONGTERM;
+#endif
 
     if (!NV_MAY_SLEEP())
     {
@@ -219,20 +196,55 @@ NV_STATUS NV_API_CALL os_lock_user_pages(
     }
 
     nv_mmap_read_lock(mm);
-    ret = NV_GET_USER_PAGES((unsigned long)address,
-                            page_count, write, force, user_pages, NULL);
-    nv_mmap_read_unlock(mm);
-    pinned = ret;
-
-    if (ret < 0)
+    ret = NV_PIN_USER_PAGES((unsigned long)address,
+                            npages, gup_flags, user_pages);
+    if (ret > 0)
     {
-        os_free_mem(user_pages);
-        return NV_ERR_INVALID_ADDRESS;
+        pinned = ret;
     }
-    else if (pinned < page_count)
+#if defined(NVCPU_FAMILY_X86) && defined(NV_FOLL_LONGTERM_PRESENT) && \
+    (defined(NV_PIN_USER_PAGES_HAS_ARGS_VMAS) ||                      \
+     defined(NV_GET_USER_PAGES_HAS_ARGS_FLAGS_VMAS))
+    //
+    // NV_PIN_USER_PAGES() passes in NULL for the vmas parameter (if required)
+    // in pin_user_pages() (or get_user_pages() if pin_user_pages() does not
+    // exist). For kernels which do not contain the commit 52650c8b466b
+    // (mm/gup: remove the vma allocation from gup_longterm_locked()), if
+    // FOLL_LONGTERM is passed in, this results in the kernel trying to kcalloc
+    // the vmas array, and since the limit for kcalloc is 4 MB, it results in
+    // NV_PIN_USER_PAGES() failing with ENOMEM if more than
+    // NV_NUM_PIN_PAGES_PER_ITERATION pages are requested on 64-bit systems.
+    //
+    // As a workaround, if we requested more than
+    // NV_NUM_PIN_PAGES_PER_ITERATION pages and failed with ENOMEM, try again
+    // with multiple calls of NV_NUM_PIN_PAGES_PER_ITERATION pages at a time.
+    //
+    else if ((ret == -ENOMEM) &&
+             (page_count > NV_NUM_PIN_PAGES_PER_ITERATION))
+    {
+        for (pinned = 0; pinned < page_count; pinned += ret)
+        {
+            npages = page_count - pinned;
+            if (npages > NV_NUM_PIN_PAGES_PER_ITERATION)
+            {
+                npages = NV_NUM_PIN_PAGES_PER_ITERATION;
+            }
+
+            ret = NV_PIN_USER_PAGES(((unsigned long) address) + (pinned * PAGE_SIZE),
+                                    npages, gup_flags, &user_pages[pinned]);
+            if (ret <= 0)
+            {
+                break;
+            }
+        }
+    }
+#endif
+    nv_mmap_read_unlock(mm);
+
+    if (pinned < page_count)
     {
         for (i = 0; i < pinned; i++)
-            put_page(user_pages[i]);
+            NV_UNPIN_USER_PAGE(user_pages[i]);
         os_free_mem(user_pages);
         return NV_ERR_INVALID_ADDRESS;
     }
@@ -255,7 +267,7 @@ NV_STATUS NV_API_CALL os_unlock_user_pages(
     {
         if (write)
             set_page_dirty_lock(user_pages[i]);
-        put_page(user_pages[i]);
+        NV_UNPIN_USER_PAGE(user_pages[i]);
     }
 
     os_free_mem(user_pages);
